#!/usr/bin/python
# Copyright 2016 CERN and GSI
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

from iecommon import *
import libxml2

class Register(object):
    
    def __init__(self, xmlNode):
        self.initWithDesignRegisterNode(xmlNode)
    
    def initWithDesignRegisterNode(self, xmlNode):
        self.xmlNode = xmlNode
        
        #xmlNode.shellPrintNode()
        self.name = xmlNode.prop("name")
        self.syncro = xmlNode.prop("synchro")
        
        valueTypes = xmlNode.xpathEval("*[name()='scalar' or name()='array' or name()='array2D' or name()='string' or name()='stringArray' or name()='stringArray2D']")
        if not valueTypes:
            iecommon.logError('ERROR: The register '+ self.name +' has no valueTypes.', True, {'errorlog': True})
        if len(valueTypes) < 1:
            iecommon.logError('ERROR: The register '+ self.name +' has no valueTypes.', True, {'errorlog': True})
        if len(valueTypes) > 1:
            iecommon.logError('ERROR: The register '+ self.name +' has multiple valueTypes.', True, {'errorlog': True})
        self.valueTypeNode = valueTypes[0]
        self.valueType = self.valueTypeNode.get_name()
        
        self.dim1 = 1
        self.dim2 = 1
        self.stringLength = 1 # ... currently needs to be default because of some old convention
        
        if self.valueTypeNode.hasProp("dim1"):
            self.dim1 = long(self.valueTypeNode.prop("dim1"))
        elif self.valueTypeNode.hasProp("dim"):
            self.dim1 = long(self.valueTypeNode.prop("dim"))
            
        if self.valueTypeNode.hasProp("dim2"):
            self.dim2 = long(self.valueTypeNode.prop("dim2"))

        if self.valueTypeNode.hasProp("string-length"):
            self.stringLength = long(self.valueTypeNode.prop("string-length"))
            
        self.format = self.valueTypeNode.prop("format")
            
    def isArray(self):
        return self.valueType == 'array' or 'stringArray'
    
    def isArray2D(self):
        return self.valueType == 'array2D' or 'stringArray2D'

#has some additionalValues
class ParamRegister(Register):
    def __init__(self):
        self.size = 0
        self.address = 0
        self.memSize = 0
            
    def initWithParamRegisterNode(self, xmlNode):
        super(ParamRegister, self).__init__(xmlNode)
        self.size = long(self.xmlNode.prop("size"))
        self.address = long(self.xmlNode.prop("address"))
        self.memSize = long(self.xmlNode.prop("mem-size"))
        
    def initWithDesignRegister(self, designRegister,size,address,memSize):
        newNodeTree = designRegister.xmlNode.copyNode(1) # 1 is for recursive copy
        if ( newNodeTree == None ):
            iecommon.logError('ERROR: Failed to copy register node: '+ designRegister.name +'.', True, {'errorlog': True})
        newNodeTree.shellPrintNode()
        super(ParamRegister, self).__init__(designRegister.xmlNode)
        self.xmlNode = newNodeTree
        newNodeTree.setProp("size", size)
        self.size = long(size)
        newNodeTree.setProp("address", str(address))
        self.address = address
        newNodeTree.setProp("mem-size", str(memSize))
        self.memSize = memSize
        
                
class DesignRegister(Register):
    def __init__(self, xmlNode):
        super(DesignRegister, self).__init__(xmlNode)
        self.fesaFieldName = ""
        if self.xmlNode.hasProp("fesaFieldName"):
            self.fesaFieldName = xmlNode.prop("fesaFieldName")
        self.generateFesaValueItem = xsdBooleanToBoolean(xmlNode.prop("generateFesaValueItem"))
    def getFesaName(self):
        if self.fesaFieldName == "":
            return self.name
        return self.fesaFieldName
