// Copyright 2016 CERN and GSI
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program.  If not, see <http://www.gnu.org/licenses/>.

#ifdef NI_SUPPORT_ENABLED

#ifndef _RECV_CNV_ACTION_H_
#define _RECV_CNV_ACTION_H_

#include <silecs-communication/interface/core/PLCAction.h>

#ifdef __cplusplus
extern "C" {
#endif

#include <nilibnetv.h>

#ifdef __cplusplus
}
#endif

namespace Silecs
{
	class Action;
	class Block;
	class Context;

	/*!
	 * \class CNVRecvBlockMode
	 * \brief Concrete action responsible to get a data block from the CNV (Block mode configuration)
	 */
	class CNVRecvBlockMode : public PLCAction
	{

	public:
		CNVRecvBlockMode(Block* block);
 		virtual ~CNVRecvBlockMode();

        /*!
         * \fn execute
         * Connect the CNV if necessary and Get data-block from its memory.
         * Overwrite the abstract method
         * \return 0 if no error occurred. else return a error status from low level library
         */
		int execute(Context* pContext);

	private:

		/* generate a string exception out of a CNV error code*/
		void errChk(int code);

	};

	/*!
	 * \class CNVRecvDeviceMode
	 * \brief Concrete action responsible to get a data block from the CNV (Device mode configuration)
	 */
	class CNVRecvDeviceMode : public PLCAction
	{

	public:
		CNVRecvDeviceMode(Block* block);
 		virtual ~CNVRecvDeviceMode();

        /*!
         * \fn execute
         * Connect the CNV if necessary and Get data-block from its memory.
         * Overwrite the abstract method
         * \return 0 if no error occurred. else return a error status from low level library
         */
		int execute(Context* pContext);

	private:

		/* generate a string exception out of a CNV error code*/
		void errChk(int code);

	};

} // namespace

#endif // _RECV_CNV_ACTION_H_

#endif //NI_SUPPORT_ENABLED
