// Copyright 2016 CERN and GSI
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program.  If not, see <http://www.gnu.org/licenses/>.

#ifndef _DIAGNOSTIC_H_
#define _DIAGNOSTIC_H_

#include <string>
#include <iostream>

namespace Silecs
{
	// Possible state of the FEC/PLC/Class connection
	static const std::string ConnectionStatusString[] =
	{ "Connected", "Disconnected", "Undefined"
	};

	typedef enum
	{ Connected, Disconnected, Undefined
	} ConnectionStatus;

	static const std::string PLCStatusString[] =
	{ "On", "Off", "Unknown"
	};

	typedef enum
	{ On, Off, Unknown
	} PLCStatus;

	/*!
	 * \class Status
	 * \brief This class is used to manage the PLC connection status
	 */
   class Status
    {
        public:
            Status();
            virtual ~Status();

            inline ConnectionStatus getConnectionStatus() { return connStatus_; }
            inline PLCStatus getPLCStatus() { return plcStatus_; }
    		inline std::string getConnectionStatusAsString() const { return ConnectionStatusString[connStatus_]; }
    		inline std::string getPLCStatusAsString() const { return PLCStatusString[plcStatus_]; }

        private:
    		friend class PLC;

    		/// Current status of the FEC/PLC/Class connection (Connected/Disconnected)
			ConnectionStatus connStatus_;
			/// Current status of the PLC (on/off)
			PLCStatus plcStatus_;
    };

} // namespace

#endif // _DIAGNOSTIC_H_
