// Copyright 2016 CERN and GSI
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program.  If not, see <http://www.gnu.org/licenses/>.

#ifndef _SILECS_ACTION_H_
#define _SILECS_ACTION_H_

#include <string>

namespace Silecs
{
	class Block;
	class Context;

	/*!
	 * \class Action
	 * \brief Action defines the abstract level of any action
	 */
	class Action
	{

	public:
		Action(Block* block) : theBlock_(block) {};
 		virtual ~Action() {};

        /*!
         * \fn execute
         * It contains the code to be executed for this action.
         * Has to be overriden for concrete PLC action.
         * \return 0 if no error occurred. else return a error status from low level library
         */
		virtual int execute(Context* pContext) = 0;

	protected:
		/// Parent block reference of that action
		Block* theBlock_;

	};

} // namespace

#endif // _SILECS_ACTION_H_

