// Copyright 2016 CERN and GSI
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program.  If not, see <http://www.gnu.org/licenses/>.

#ifndef _CONDITION_H_
#define _CONDITION_H_

#include <pthread.h>
#include <errno.h>

#include <silecs-communication/interface/utility/Mutex.h>

namespace Silecs {

	/*!
	 * \class Condition
	 * \brief This class implement the basic methods to manage the thread synchronization
	 * with Conditional Variable. In particular, it manages the related mutex.
	 */
	class Condition
	{
		public:
			Condition(std::string name = "undefined");
			virtual ~Condition();

	        void lock();
	        void unlock();

			void signal();
			void broadcast();
			void wait();

		private:
			// Condition name if any, used for logging in particular
			std::string name_;

			pthread_cond_t condVar_;
			Mutex* pMutex_;
	};

} // namespace

#endif /* _CONDITION_H_ */
