// Copyright 2016 CERN and GSI
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program.  If not, see <http://www.gnu.org/licenses/>.

#include <silecs-communication/wrapper/Controller.h>
#include <silecs-communication/wrapper/Design.h>

namespace SilecsWrapper
{

Controller::Controller(const std::string& name, const std::string& domain, Design *design, const std::string parameterFile) :
                _name(name),
                _domain(domain),
                _silecsPLC(design->getSilecsCluster()->getPLC(_name, parameterFile))
{

}

Controller::~Controller()
{

}

void Controller::connect()
{
    //By default: Controller connected right-away and Master registers download.
    _silecsPLC->connect(Silecs::MASTER_SYNCHRO, true);
}

void Controller::connect(Silecs::SynchroMode synchroMode, bool connectNow)
{
    //Connect parameters can be set in manual connection mode (Automatic connect==false)
    _silecsPLC->connect(synchroMode, connectNow);
}

void Controller::disconnect()
{
    _silecsPLC->disconnect();
}

} //namespace SilecsWrapper
